Scriptname REQ_AttributeSystem extends REQ_PlayerAlias
{This script controls the derived attributes, they are updated each time the skills menu is closed.
The contributions are computed as prefactor * sqrt( weighted base attribute sum - threshold). Negative 
arguments to the square root have no contribution.}

;parameters, to be defined in the esp
String[] Property derived Auto
{list of derived stats to mod, attackdamagemult must come before any weaponskillmods!}
Float[] Property prefactor Auto
{scaling prefactors for the attributes}
Float[] Property weight_health Auto
{weighting factors for base health, weights should sum to 1}
Float[] Property weight_magicka Auto
{weighting factors for base magicka, weights should sum to 1}
Float[] Property weight_stamina Auto
{weighting factors for base stamina, weights should sum to 1}
Float[] Property threshold Auto
{minimum value the weighted attribute sum must reach to give any bonus to the specified derived stat}
Bool[] Property rescale Auto
{flag this attribute as a weapon damage one, as such it must be rescaled according to the attackdamagemult,
because some weapons are not properly linked to the respective attributes (daggers)}
String[] Property shout_derived Auto
{the individual components (game misc stats) that make up the shout cooldown reduction}
Float[] Property prefactor_shouts Auto
{the weighting prefactors for the shout cooldown bonus components, these should be negative}
Float[] Property shout_thresholds Auto
{the minimum value the misc stat must have to contribute at all}
GlobalVariable Property do_once Auto
{the global variable used by the starting perks alias to indicate that one-time inits (spell-removal,
free perks, stats adjustment) have been finished}

;runtime variables with external access (MCM display/definition in esp)
Float[] Property Modifiers Auto 
{the current computed actorvalue modifiers, please define as array with the same size as "derived"
with 0 values}
Float[] Property Mods_display Auto
{the display modifiers for the MCM, partially different from Modifiers to the special treatment for
"attackdamagemult", please define as array with the same size as "derived" with 0 values}
Float[] Property Mods_Shout Auto
{the current shout modifiers, both for internal usage and MCM display, please define as array with
the same size as "shout_derived" with 0 values}

Bool doagain = False
Bool shortcut = False
Bool Updating = True

REQ_MCM_Skills Property mcm Auto

Event ScriptShutDown(string eventName, string strArg, float numArg, Form sender)
	Updating = True
	While (GetState() != "")
		Utility.Wait(0.5)
	EndWhile
	Player.ModAV("ShoutRecoveryMult", -0.5)
	UnregisterForAllMenus()
	Reset_Buffs()
	REQ_AutoUpdater UpdateControl = GetOwningQuest().GetAlias(0) as REQ_AutoUpdater
	Int index = UpdateControl.Scripts.find(self)
	UpdateControl.ShutdownReady[index] = True
EndEvent

Event ScriptInit(string eventName, string strArg, float numArg, Form sender)
	Player.ModAV("ShoutRecoveryMult", 0.5)
	mcm.modname = "$REQ_modname_skills"
	mcm.pages = new String[2]
	mcm.pages[0] = "$REQ_cat_skill0"
	mcm.pages[1] = "$REQ_cat_skill1"
	mcm.player = Player
	Reset_Buffs()
	While do_once.GetValueInt() == 0
		Utility.Wait(0.5)
	EndWhile
	Evaluate_Attributes()
	Evaluate_Shouts()
	RegisterForMenu("StatsMenu")
	RegisterForMenu("MagicMenu")
	REQ_AutoUpdater UpdateControl = GetOwningQuest().GetAlias(0) as REQ_AutoUpdater
	Int index = UpdateControl.Scripts.find(self)
	Updating = False
	UpdateControl.ShutdownReady[index] = False
EndEvent

Event OnPlayerLoadGame()
	;this should be unnecessary and is a security addition
	UnregisterForAllMenus()
	RegisterForMenu("StatsMenu")
	RegisterForMenu("MagicMenu")
EndEvent

Event OnMenuClose(String Menu)
	GoToState("working")
	If Menu != "MagicMenu"
		Evaluate_Attributes()
	EndIf
	Evaluate_Shouts()
	GoToState("")
	If doagain
		doagain = False
		OnMenuClose("")
	EndIf
EndEvent

State working
	Event OnMenuClose(String Menu)
		doagain = True
	EndEvent
EndState

Function Reset_Buffs()
	Int i = 0
	While i < derived.length
		Player.ModAV(derived[i], -Modifiers[i])
		Modifiers[i] = 0.0
		Mods_display[i] = 0.0
		i += 1
	EndWhile
	Player.ModAV("ShoutRecoveryMult", -Mods_Shout[0])
	i = 0
	While i < Mods_Shout.length
		Mods_Shout[i] = 0
		i += 1
	EndWhile
EndFunction

Function Evaluate_Attributes()
	Int i = 0
	int j = derived.find("AttackDamageMult")
	Float magicka = Player.GetBaseAV("Magicka")
	Float stamina = Player.GetBaseAV("Stamina")
	Float health = Player.GetBaseAV("Health")
	Float weighted = 0.0
	While i < derived.length
		Player.ModAV(derived[i], -Modifiers[i])
		weighted = weight_health[i] * health + weight_magicka[i] * magicka + weight_stamina[i] * stamina
		If weighted > threshold[i]
			Modifiers[i] = prefactor[i] * Math.Pow(weighted - threshold[i], 0.5)
			Mods_display[i] = Modifiers[i]
		Else
			Modifiers[i] = 0.0
			Mods_display[i] = 0.0
		EndIf
		If rescale[i]
			Modifiers[i] = 100 / (1 + Modifiers [j] ) + Modifiers[i] / (1 + Modifiers[j] ) - 100
		EndIf
		Player.ModAV(derived[i], Modifiers[i])
		i += 1
	EndWhile
EndFunction

Function Evaluate_Shouts()
	int i = 0
	Float statvalue = 0.0
	Player.ModAV("ShoutRecoveryMult", -Mods_Shout[0])
	While i < shout_derived.length
		statvalue = Game.QueryStat(shout_derived[i]) as Float
		If statvalue > Shout_Thresholds[i]
			Mods_Shout[i+1] = prefactor_shouts[i] * Math.Pow(statvalue - Shout_Thresholds[i], 0.5)
		Else
			Mods_Shout[i+1] = 0.0
		EndIf
		i += 1
	EndWhile
	Mods_Shout[0] = 0.0
	i = 1
	While i < Mods_Shout.length
		Mods_Shout[0] = Mods_Shout[0] - Mods_Shout[i]
		i += 1
	EndWhile
	Player.ModAV("ShoutRecoveryMult", Mods_Shout[0])
EndFunction
