Scriptname REQ_QuestRestarter extends Quest
{script state manager for player-ref scripts quest}

ReferenceAlias Property AutoUpdater Auto
{ref alias with AutoUpdater script attached}
Actor Property Stalker Auto
{the invisible and invincible guy following you all the time}
Message Property Install Auto
Message Property SKSE_missing Auto
{message to display if SKSE is missing at all or too old}
Message Property SKSE_broken Auto
{message if SKSE dll and script versions differ}
Message Property NoNewGame Auto
{message if Requiem was installed into an existing non-Requiem save game}
Message Property UpgradeForbidden Auto
{message if Requiem was installed into an existing pre-1.8 Requiem save game}
Message Property DoubleUpdate Auto
{message if Requiem tries to start a second script update in parallel}
Message Property SkyProcMissing Auto
{message if Requiem for the Indifferent is missing and the data is therefore garbage}
Message Property SkyProcOutdated Auto
{message if Requiem for the Indifferent is based on an older Requiem version than installed}
Message Property PCEApatchinvalid Auto
{message if Requiem finds the broken PCEA "patch" (clone, not patch) in the load order}
Message Property BashedPatchFailed Auto
{message if (most likely) a bashed patch has broken the SkyProc-processed levelled lists}

GlobalVariable Property VersionActive Auto
{GV indicating the currently active Requiem version (in savegame}
GlobalVariable Property VersionInstalled Auto
{GV indicating the currently installed Requiem version (static in esp)}
GlobalVariable Property VersionIndifferent Auto
{GV indicating the Requiem version the SkyProc patch is based on}
GlobalVariable Property KillSwitchOverride Auto
{constnat GV to temporarily override the killswitch in the update checks, for debug intended only}
LeveledItem Property TestList Auto
{a skyproc-processed list, if a bashed patch has the wrong tags, it will have only 1 entry}

Spell Property HealthRegenStop Auto
{Magic ability on all playable races that should stop natural health regeneration}
GlobalVariable Property RaceWarnOverride Auto
{GV indicating that overwritten race warnings should not be displayed}
Message Property RaceWarning Auto
{message if Requiem for the Indifferent is based on an older Requiem version than installed}

;stops the quest, wait a few seconds and restart it to update all alias scripts
Function Restart()
	Self.Stop()
	SetupCheck()
	Utility.Wait(1)
	Self.Start()
	Utility.Wait(1)
	(AutoUpdater as REQ_AutoUpdater).InitScripts()
EndFunction

Event OnStoryChangeLocation(ObjectReference akActor, Location akOldLocation, Location akNewLocation)
	If (akOldLocation != None)
		Install.Show()
		SetupCheck()
		Utility.Wait(1)
		(AutoUpdater as REQ_AutoUpdater).InitScripts()
	Else
		Self.Stop()
	EndIf
EndEvent

; check whether the current setup is suitable for Requiem, if not display a warning and quit to menu
Bool Function SetupCheck()
	bool valid = true
	Int version_skse = SKSE.GetVersionRelease()
	Int version_script = SKSE.GetScriptVersionRelease()
	Int count = 0
	int req_index = Game.GetModByName("Requiem.esp")
	int pcea_clone = Game.GetModByName("Requiem plus PCEA 3-5.esp")
	Bool dependent = False
	; SKSE installation check
	If (version_skse < 46)
		SKSE_missing.Show()
		valid = false
	ElseIf (version_skse != version_script )
		SKSE_broken.Show(version_skse as Float, version_script as Float)
		valid = false
	ElseIf (Utility.GetCurrentGameTime() > 0.5 && VersionActive.GetValueInt() == 0)
	; new game check
		NoNewGame.Show()
		valid = false
	ElseIf (VersionActive.GetValueInt() == -1)
	; no double update (save during init/update + later reload)
		DoubleUpdate.Show()
		valid = false
	ElseIf (VersionActive.GetValueInt() > 0 && VersionActive.GetValueInt() < 18000)
	; unsupported upgrading path
		UpgradeForbidden.Show()
		valid = false
	ElseIf Game.GetModByName("Requiem for the Indifferent.esp") == 255
	; SkyProc patch missing
		SkyProcMissing.Show()
		valid = false
	ElseIf VersionInstalled.GetValueInt() != VersionIndifferent.GetValueInt()
	; SkyProc patch outdated
		SkyProcOutdated.Show()
		valid = false
	ElseIf pcea_clone != 255
	; check for broken PCEA "patch"...
		count = Game.GetModDependencyCount(pcea_clone)
		While count > 0 && !dependent
			count = count - 1
			dependent = req_index == Game.GetNthModDependency(pcea_clone, count)
		EndWhile
		If !dependent
			valid = false
			PCEApatchinvalid.Show()
		EndIf
	ElseIf TestList.GetNumForms() == 1
	; bashed patch with wrong tags for Requiem for the indifferent
		BashedPatchFailed.Show()
		valid = false
	EndIf
	CheckRaces()
	If valid
		RegisterForSingleUpdate(30)
		return true
	ElseIf KillSwitchOverride.getValueInt() != 1
		Game.QuitToMainMenu()
	Else
		return false
	EndIf
EndFunction

Bool Function CheckRaces()
	int count_r = Race.GetNumPlayableRaces()
	int count_s = 0
	Race currentrace
	bool valid = False
	While count_r > 0
		count_r += -1
		currentrace = Race.GetNthPlayableRace(count_r)
		valid = False
		count_s = currentrace.GetSpellCount()
		While count_s > 0 && !valid
			count_s += -1
			If currentrace.GetNthSpell(count_s) == HealthRegenStop
				valid = True
			EndIf
		EndWhile
		If !valid && RaceWarnOverride.GetValueInt() != 1
			RaceWarning.Show()
			return False
		EndIf
	EndWhile
	return True
EndFunction

Event OnUpdate()
	(AutoUpdater as REQ_AutoUpdater).CheckKillswitch()
EndEvent
