Scriptname REQ_IllusionBreak extends ActiveMagicEffect
{periodically checks, if the target breaks free from a mind-influence effect before its natural end}

;Default Settings, exposed in case specific adjustments are necessary
Float Property UpdateInterval = 2.0 Auto
{time between consecutive tests, default = 2}
Float Property Mult_Level = 25.0 Auto
{prefactor for the square of the target's level, default=25}
Float Property Mult_Magicka = 2.0 Auto
{prefactor for the square of the target's max magicka, default=2}
Float Property Mult_Illusion = 20.0 Auto
{prefactor for the square of the target's illusion skill, default=20}
Float Property Mult_MagicResist = 0.01 Auto
{prefactor for the square of the target's magic resistance, default=20}
Keyword Property MindFogEffect Auto
{Effect Keyword of the mind fog spell}

;Spell-specific settings
Perk[] Property Skill_ScalingPerks Auto
{each perk in this list will increase the scaling multiplicator by 1}
Float Property XPRate Auto
{the amount of illusion XP the caster gains each time the target fails to break the spell}
Float Property Skill_Offset Auto
{added to the casters illusion skill to determine effective skill}
Spell Property BaseSpell Auto
{the unique spell which owns this effect, is required to break free from all effects simultaneously}
String Property CasterSkill = "Illusion" Auto
{determines the skill to be used by the caster for the resistance check, default = Illusion}
Spell Property MindEffect Auto
{the actual mind effect, seperated to avoid possible exploits}
Bool Property InverseFunction = False Auto
{invert the script function, the spell is dispelled once it was NOT resisted}
Bool Property SingleResist = False Auto
{instead of continuous resistances allow only a single check at the beginning}
Bool Property NoAssault = False Auto
{True if the spell is not considered hostile if failed}

Float chance = 0.0
Bool grantxp = False

Event OnEffectStart(Actor akTarget, Actor akCaster)
	; Float temp = (akTarget.GetAV("Magicka")*1000)/(akTarget.GetAVPercentage("Magicka")*1000)
	Float temp = akTarget.GetBaseAV("Magicka")
	Float random = Utility.RandomFloat(0,100)
	Int i = 0
	chance = Mult_Level * akTarget.GetLevel() * akTarget.GetLevel()
	chance += Mult_Magicka * temp * temp
	chance += Mult_Illusion * akTarget.GetAV("Illusion") * akTarget.GetAV("Illusion")
	temp = 1.0
	While (i < Skill_ScalingPerks.Length)
		temp += akCaster.HasPerk(Skill_ScalingPerks[i]) as Float
		i += 1
	EndWhile
	temp *= (akCaster.GetAV(CasterSkill)+Skill_Offset )*(akCaster.GetAV(CasterSkill)+Skill_Offset )
	chance /= temp
	chance += Math.abs(akTarget.GetAV("MagicResist"))*akTarget.GetAV("MagicResist")/100.0
	chance /= 1.0 + 0.5 * akTarget.HasMagicEffectWithKeyword(MindFogEffect) as Float
	; Debug.MessageBox("init: "+random+" / "+chance+"\ncaster: "+akCaster)
	XPRate *=  (akTarget.GetLevel() as float)/10.0
	If ( (random < chance && !InverseFunction) || (random > chance && InverseFunction) )
		; Debug.MessageBox("stop spell")
		If (Basespell != None)
			GetTargetActor().DispelSpell(BaseSpell)
		Else
			Self.Dispel()
		EndIf
		If !NoAssault
			akTarget.SendAssaultAlarm()
		EndIf
	Else
		; Debug.MessageBox("init spell "+MindEffect)
		If ( MindEffect != None )
			MindEffect.Cast(akCaster, akTarget)
		EndIf
		grantxp = ( akCaster == Game.GetPlayer() && XPRate > 0)
		If ( grantxp )
			Game.AdvanceSkill("Illusion", XPRate)
		EndIf
		If (!singleresist)
			RegisterForSingleUpdate(UpdateInterval)
		EndIf
	EndIf
EndEvent

Event OnUpdate()
	float random = Utility.RandomFloat(0,100)
	; Debug.MessageBox(random+" / "+chance)
	If ( ((random < chance && !InverseFunction) || (random > chance && InverseFunction) ) )
		If (MindEffect != None)
			GetTargetActor().DispelSpell(MindEffect)
		EndIf
		If (Basespell != None)
			GetTargetActor().DispelSpell(BaseSpell)
		Else
			Self.Dispel()
		EndIf
	Else
		RegisterForSingleUpdate(UpdateInterval)
		If ( grantxp )
			Game.AdvanceSkill("Illusion", XPRate)
		EndIf
	EndIf
EndEvent

Event OnEffectFinish(Actor akTarget, Actor akCaster)
	; Debug.MessageBox("effect terminated, chance: "+chance)
	If (MindEffect != None)
		akTarget.DispelSpell(MindEffect)
	EndIf
	If (Basespell != None)
		akTarget.DispelSpell(BaseSpell)
	EndIf
EndEvent
